"""EnvProxy creates a proxy to environmental variables with typehinting and type conversion.
"""

import os
from typing import Any, Optional


class EnvProxy:
    """A proxy to environmental variables with typehinting and type conversion"""

    env_prefix: Optional[str] = None

    @classmethod
    def _get_key(cls, key: str) -> str:
        """Get key with prefix"""
        prefix = f"{cls.env_prefix}_" if cls.env_prefix else ""
        return f"{prefix}{key.upper().replace('-', '_')}"

    @classmethod
    def get_any(cls, key: str) -> Any:
        """Get any value"""
        key_cleaned = cls._get_key(key)
        val = os.getenv(key_cleaned, None)
        return val

    @classmethod
    def get_str(cls, key: str) -> Optional[str]:
        """Get str value from environment"""
        val = cls.get_any(key)
        if val is None:
            return None
        if isinstance(val, str):
            return val
        return str(val)

    @classmethod
    def get_str_strict(cls, key: str) -> str:
        """Get str value from environment, raise ValueError if no such key exists"""
        val = cls.get_str(key)
        if val is None:
            raise ValueError(f"No value for key {key} in environment")
        return val

    @classmethod
    def get_int(cls, key: str) -> Optional[int]:
        """Get int value from environment"""
        val = cls.get_any(key)
        if val is None:
            return None
        if isinstance(val, int):
            return val
        try:
            return int(val)
        except ValueError as error:
            raise ValueError(f"Value for key {key} is not a valid integer") from error

    @classmethod
    def get_int_strict(cls, key: str) -> int:
        """Get int value from environment, raise ValueError if no such key exists"""
        val = cls.get_int(key)
        if val is None:
            raise ValueError(f"No value for key {key} in environment")
        return val

    @classmethod
    def get_float(cls, key: str) -> Optional[float]:
        """Get float value from environment"""
        val = cls.get_any(key)
        if val is None:
            return None
        if isinstance(val, float):
            return val
        try:
            return float(val)
        except ValueError as error:
            raise ValueError(f"Value for key {key} is not a valid integer") from error

    @classmethod
    def get_float_strict(cls, key: str) -> float:
        """Get float value from environment, raise ValueError if no such key exists"""
        val = cls.get_float(key)
        if val is None:
            raise ValueError(f"No value for key {key} in environment")
        return val



# -*- coding: utf-8 -*-
from setuptools import setup

modules = \
['env_proxy']
setup_kwargs = {
    'name': 'env-proxy',
    'version': '0.1.1',
    'description': 'Creates a class used to query environmental variables with typehinting a conversion to basic Python types.',
    'long_description': '# EnvProxy\n\n`EnvProxy` provides a class used to query environmental variables with typehinting a conversion to basic Python types.\nYou can query your environment easily and keep your typehinting.\n\n## Installation\n\nUsing `pip`:\n\n```console\npip install env-proxy\n```\n\nUsing `poetry`:\n\n```console\npoetry add env-proxy\n```\n\n## Example\n\n```python\n# Import EnvProxy\nfrom env_proxy import EnvProxy\n\n# Basic examples\n## Environment variable "DATABASE_HOST"\ndatabase_host = EnvProxy.get_str("database-host")\n\n## If you want the function to fail if the value does not exist, use methods with `_strict` suffix\ndatabase_nonsene = EnvProxy.get_str_strict("database-nonsense")\n### ValueError: No value for key DATABASE_NONSENSE in environment\n\n## Specify default for the (non-zero) variable "DATABASE_PORT"\ndatabase_port = EnvProxy.get_int("database-port") or 5432\n\n# Specify custom prefix\nclass MyProxy(EnvProxy):\n    env_prefix: Optional[str] = "MYAPP"\n## Now all variables are expected to be prefixed with MYAPP_\ndatabase_host = EnvProxy.get_str("database-host")\n### Searches for MYAPP_DATABASE_HOST variable\n```\n\n## Documentation\n\nSee [docs](https://tomasvotava.github.io/env-proxy/)\n',
    'author': 'Tomas Votava',
    'author_email': 'info@tomasvotava.eu',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/tomasvotava/env-proxy',
    'py_modules': modules,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
